* {
    margin: 0;
    padding: 0;
    box-sizing: border-box;
}

html {
    scroll-behavior: smooth;
}

html * {
    scroll-margin-top: 8rem;
}

h2>.spinner-border {
    --bs-spinner-border-width: 0.2em;
}

/* small */
@media (max-width: 992px) {
    #navbar-menu-actions {
        width: 100%;
    }

    #navbar-menu-actions .dropdown-menu {
        position: absolute;
        left: 0;
        top: 100%;
        width: 100%;
        transform: translateY(0);
    }
}

/* large */
@media (min-width: 992px) {
    #navbar-menu-actions {
        width: auto;
    }

    .form-actions>div {
        padding-left: 0;
    }
}

#navbar-menu-actions button {
    white-space: nowrap;
}

#navbar-menu-actions button i {
    width: 1rem;
    text-align: center;
    margin-right: 0.5rem;
}

.dropdown-item:hover,
.dropdown-item:hover i {
    color: var(--bs-white) !important;
}

.menu-primary:hover {
    background-color: var(--bs-primary);
    /* color: var(--bs-white); */
}

.menu-success:hover {
    background-color: var(--bs-success);
    /* color: var(--bs-white); */
}

.menu-danger:hover {
    background-color: var(--bs-danger);
    /* color: var(--bs-white); */
}

.btn-close {
    transform: scale(0.75);
}

#profile-container .btn-group {
    white-space: nowrap;
}

#config-form-edit .editor-check {
    padding-left: 0;
}

#config-form-edit .editor-check-input {
    position: relative;
}

#config-form-edit .check-input-plain {
    top: .3rem;
}

#config-form-edit .check-input-heading {
    top: .4rem;
}

#config-form-edit .je-switcher {
    padding: .25rem 2.25rem .25rem .75rem !important;
    font-style: normal;
    margin-bottom: .25rem !important;
}

#config-form-edit .je-object__controls {
    top: -.15rem;
}

#config-form-edit .card-title {
    margin-bottom: 0rem;
}

#config-form-edit .level-1 {
    margin-bottom: 0.2rem;
}

#config-form-edit .user-add-item {
    display: inline-block;
    font-weight: bolder;
    margin-right: .75rem;
}

.code-text {
    font-family: monospace;
}

#json-code-edit {
    min-height: 40rem;
}

#json-code-content-placeholder button {
    height: 40rem;
    color: var(--bs-secondary);
}

#json-code-content-placeholder span {
    font-weight: bold;
}

#terminal-output-display {
    min-height: 30rem;
}